<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use DateTime;

class DadosDoPrato extends \PME\Services\TeknisaCrud {
    protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $elaboracaoCardapio;
	protected $utilitiesCard;
	protected $comensalEstimadoCardapio;

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $elaboracaoCardapio, $utilitiesCard, $comensalEstimadoCardapio) {
        parent::__construct($crudService, $parameterBag);
        $this->connection               = $connection;
        $this->newCode                  = $newCode;
        $this->environment              = $environment;
        $this->entityManager            = $entityManager;
        $this->elaboracaoCardapio       = $elaboracaoCardapio;
        $this->utilitiesCard            = $utilitiesCard;
        $this->comensalEstimadoCardapio = $comensalEstimadoCardapio;
	}
	
	/* ------------------------ VALIDAR SE OS PRODUTOS DA RECEITA ESTÃO ATIVOS ------------------------*/
	
	public function recipeValidate(DTO\Request\Row $request, DTO\Response $response) {
	    $row = $request->getRow();
	    // Verificar: se sabemos qual a receita não seria melhor rodar apenas a parte da query do tipo de receita
	    $sql = "SELECT FORMATPRODMASK(P.CDARVPROD) || ' - ' || P.NMPRODUTO AS DESCRICAO, P.CDARVPROD, P.NMPRODUTO, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV FROM   RECESERV R INNER JOIN PRODUTO P ON P.CDNVPRODUTO = :P_CDNVPRODUTO AND P.CDPRODUTO = R.CDPRODUTO LEFT JOIN PRODFILI PF ON R.CDFILIAL = PF.CDFILIAL AND P.CDPRODUTO = PF.CDPRODUTO WHERE R.CDPRATO = :P_CDPRATO AND R.CDFILAUXPRAT = :P_CDFILAUXPRAT AND R.CDFILIAL     = :P_CDFILIAL     AND R.CDSERVFILI       = :P_CDSERVFILI AND 'S' = :P_TPREC UNION ALL SELECT FORMATPRODMASK(P.CDARVPROD) || ' - ' || P.NMPRODUTO AS DESCRICAO, P.CDARVPROD, P.NMPRODUTO, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV FROM   RECETPCARD R INNER JOIN PRODUTO P ON P.CDNVPRODUTO = :P_CDNVPRODUTO AND P.CDPRODUTO = R.CDPRODUTO LEFT JOIN PRODFILI PF ON :P_CDFILIAL = PF.CDFILIAL AND P.CDPRODUTO = PF.CDPRODUTO WHERE R.CDPRATO = :P_CDPRATO AND R.CDFILAUXPRAT = :P_CDFILAUXPRAT AND R.CDFILCENTPME = :P_CDFILCENTPME AND R.CDTIPOCARD    = :P_CDTIPOCARD AND 'T' = :P_TPREC UNION ALL SELECT FORMATPRODMASK(P.CDARVPROD) || ' - ' || P.NMPRODUTO AS DESCRICAO, P.CDARVPROD, P.NMPRODUTO, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV FROM   RECEFILI R INNER JOIN PRODUTO P ON P.CDNVPRODUTO = :P_CDNVPRODUTO AND P.CDPRODUTO = R.CDPRODUTO LEFT JOIN PRODFILI PF ON R.CDFILIAL = PF.CDFILIAL AND P.CDPRODUTO = PF.CDPRODUTO WHERE R.CDPRATO = :P_CDPRATO AND R.CDFILAUXPRAT = :P_CDFILAUXPRAT AND R.CDFILIAL = :P_CDFILIAL AND 'F' = :P_TPREC UNION ALL SELECT FORMATPRODMASK(P.CDARVPROD) || ' - ' || P.NMPRODUTO AS DESCRICAO, P.CDARVPROD, P.NMPRODUTO, P.IDPRODATIVPR, NVL(PF.IDPRODATIV, 'A') AS IDPRODATIV FROM   RECEPADR RP INNER JOIN PRODUTO  P ON P.CDNVPRODUTO = :P_CDNVPRODUTO AND P.CDPRODUTO = RP.CDPRODUTO LEFT JOIN PRODFILI PF ON :P_CDFILIAL = PF.CDFILIAL AND P.CDPRODUTO = PF.CDPRODUTO WHERE RP.CDPRATO = :P_CDPRATO AND RP.CDFILAUXPRAT = :P_CDFILAUXPRAT AND 'P' = :P_TPREC";
		$paramSelect = array(
            "P_CDNVPRODUTO"     => $row['CDNVPRODUTO'],
            "P_CDPRATO"         => $row['CDPRATO'],
            "P_CDFILAUXPRAT"    => $row['CDFILAUXPRAT'],
            "P_CDFILIAL"        => $row['CDFILIAL'],
            "P_CDSERVFILI"      => $row['CDSERVFILI'],
            "P_TPREC"           => $row['IDRECEITA'],
            "P_CDFILCENTPME"    => $row['CDFILCENTPME'],
            "P_CDTIPOCARD"      => $row['CDTIPOCARD']
        );
        
        $recipe = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        $response->addDataSet(new DataSet("recipe", $recipe));
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	public function getCardiaOnClickPlate(DTO\Request\Row $request, DTO\Response $response) {
		try{
			$this->connection->beginTransaction();

			$row = $request->getRow();
			if ($row['NRCARDIAPME'] == NULL) {
				$newCardia = $this->elaboracaoCardapio->insertInCardia($row);
			} else {
				$newCardia = array(array('QTCOMESTCPME' => $row['QTCOMESTCPME'], 'DTCARDIAPME' => $row['DTCARDIAPME']));
			}

			$this->connection->commit();
			$response->addDataSet(new DataSet("cardiaOnClickPlate", $newCardia));
		} catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	
	/* ------------------------ INSERE UM NOVO PRATO ------------------------*/
	
	public function newPlate(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$hasSaved = false;
		$hasPlates = null;
		
		try {
			$this->connection->beginTransaction();
			foreach ($row['plateData'] as $plateData) {
				if ($plateData[0]['NRCARDIAPME'] == NULL || $plateData[0]['filterData']['aggregated'] != NULL) {
					$newCardia = $this->elaboracaoCardapio->insertInCardia($plateData[0]);
					
					if(!empty($newCardia)) {
						$plateData[0]['CDFILCENTPME'] = $newCardia[0]['CDFILCENTPME'];
						$plateData[0]['NRCARDPME'] = $newCardia[0]['NRCARDPME'];
						$plateData[0]['NRCARDIAPME'] = $newCardia[0]['NRCARDIAPME'];
						$plateData[0]['CDSERVFILI'] = $newCardia[0]['CDSERVFILI'];
						$plateData[0]['DTCARDIAPME'] = $newCardia[0]['DTCARDIAPME'];
		
						if ($newCardia[0]['QTCOMESTCPME'] != 0) {
							if ($plateData[0]['filterData']['aggregated'] != NULL) $hasPlates = $this->hasPlateDay($plateData[0]);
							
							if ($plateData[0]['VRPECONSCPME'] == null) $plateData[0]['VRPECONSCPME'] = 100;
							$this->insertNewPlate($plateData, $row);
							
							if ($plateData[0]['filterData']['aggregated'] != NULL and !$hasPlates) {
								// insere demais pratos do cardápio da centrla no card específico
								$cardiaCentral = $this->elaboracaoCardapio->checkCardInDay($plateData[0], $newCardia[0]['CDFILCENTPME']);
								if (count($cardiaCentral) > 0) {
									$plateCentralInsert = $this->getPlateCentralInsertEsp(
										$cardiaCentral[0], $plateData[0]['CDTIPOPRAT'], 
										$plateData[0]['NRSEQCNFCPME'], $plateData[0]['NRCARDIAPME']
									);
									
									if(count($plateCentralInsert) > 0) {
										$selectedPlate = array();
										foreach ($plateCentralInsert as $plates) {
											$plates['NRCARDIAPME']  = $plateData[0]['NRCARDIAPME'];
											// $plates['VRPECONSCPME'] = $plateData[0]['VRPECONSCPME'];
											$vrpeconscpm = $this->getPlateConsCPme($plates, $plateData[0]);
											// 100 * (1 - (PP.VRPERACEIT/100) * -1)
											$plates['VRPECONSCPME'] = !empty($vrpeconscpm) ? (100 * (1 - ($vrpeconscpm['VRPERACEIT'] / 100) * -1)) : $plates['VRPECONSCPME'];
											$plates['NRSEQCNFCPME'] = $plates['NRSEQPRACPME'];
											$plates['NRORDCNFCPME'] = $plates['NRORDPRACPME'];
											
											$selectedPlate['CDFILAUXPRAT'] = $plates['CDFILAUXPRAT'];
											$selectedPlate['CDPRATO'] = $plates['CDPRATO'];
											
											$this->insertPlate($plates, $selectedPlate);
										}
									}
								}
							}
						}
					}
			    } 
			    else $this->insertNewPlate($plateData, $row);
			}
			$this->connection->commit();
			$response->addDataSet(new DataSet("plateSaved", array(true)));
		} catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getPlateConsCPme($plate, $plateData){
		$sql = "SELECT VRPERACEIT FROM PERACEITPRATO WHERE CDFILCENTPME = :CDFILCENTPME AND CDFILIAL = :CDFILIAL AND CDTIPOCARD = :CDTIPOCARD AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT";
		$params = array(
            "CDFILCENTPME" => $plate['CDFILCENTPME'],
            "CDFILIAL" => $plateData['CDFILIAL'],
            "CDTIPOCARD" => $plateData['filterData']['typeMenu']['CDTIPOCARD'],
            "CDPRATO" => $plate['CDPRATO'],
            "CDFILAUXPRAT" => $plate['CDFILAUXPRAT'],
		);
        return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
	}
	
	private function hasPlateDay($plateData) {
		$sql = "SELECT CDFILCENTPME FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $plateData['CDFILCENTPME'],
            "P_NRCARDPME"    => $plateData['NRCARDPME'],
            "P_NRCARDIAPME"  => $plateData['NRCARDIAPME'],
            
		);
        return $this->entityManager->getConnection()->fetchColumn($sql, $paramSelect);
	}
	
	private function getPlateCentralInsertEsp($cardiaCentral, $tpPlate, $seq, $cardEsp){
		$sql = "SELECT C.CDFILCENTPME, C.NRCARDPME, C.NRCARDIAPME, C.NRSEQPRACPME,
				       C.CDFILAUXPRAT, C.CDPRATO, C.CDTIPOPRAT, C.VRPECONSCPME,
				       C.NRORDPRACPME
				  FROM COMPCARDPME C
				 WHERE C.CDFILCENTPME = :P_CDFILCENTPME
				   AND C.NRCARDPME = :P_NRCARDPME
				   AND C.NRCARDIAPME = :P_NRCARDIAPME
				   AND C.CDTIPOPRAT <> :P_CDTIPOPRAT
				   AND C.NRSEQPRACPME <> :P_NRSEQPRACPME
				   AND NOT EXISTS (SELECT E.CDFILCENTPME, E.NRCARDPME, E.NRCARDIAPME, E.NRSEQPRACPME
									  FROM COMPCARDPME E
									 WHERE E.CDFILCENTPME = C.CDFILCENTPME
									   AND E.NRCARDPME = C.NRCARDPME
									   AND E.NRCARDIAPME = :P_NRCARDIAPME_ESP
									   AND E.CDTIPOPRAT = C.CDTIPOPRAT
									   AND E.NRSEQPRACPME = C.NRSEQPRACPME)";
								      
		$paramSelect = array(
            "P_CDFILCENTPME"    => $cardiaCentral['CDFILCENTPME'],
            "P_NRCARDPME"       => $cardiaCentral['NRCARDPME'],
            "P_NRCARDIAPME"     => $cardiaCentral['NRCARDIAPME'],
            "P_CDTIPOPRAT"      => $tpPlate,
            "P_NRSEQPRACPME"    => $seq,
            "P_NRCARDIAPME_ESP" => $cardEsp
        );
    	
    	return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function insertNewPlate($plateData, $row) {
	    $config = $this->getConfig($plateData[0]);
	    if(count($config) > 0) {
			$hasPlate = $this->hasPlate($plateData[0]);
		    if ($plateData[0]['filterData']['aggregated'] != null) $this->updateCardia($plateData[0]);
			if(count($hasPlate) > 0) {
				$this->updatePlate($plateData[0], $row['selectedPlate']);
			} else {
			    $this->insertPlate($plateData[0], $row['selectedPlate']);
			}
		}
	}
	
	private function insertPlate($plateData, $selectedPlate) {
		$cdfilcentpme = $plateData['CDFILCENTPME'];
		$nrcardpme = $plateData['NRCARDPME'];
		$nrcardiapme = $plateData['NRCARDIAPME'];
		$nrseqcnfcpm = $plateData['NRSEQCNFCPME'];
		$cdfilauxprat = $selectedPlate['CDFILAUXPRAT'];
		$cdprat = $selectedPlate['CDPRATO'];
		$cdtipoprat = $plateData['CDTIPOPRAT'];
		$vrpeconscpm = $plateData['VRPECONSCPME'];
		$nrordcnfcpme = $plateData['NRORDCNFCPME'];
		
		$this->utilitiesCard->insertCompCard($cdfilcentpme, $nrcardpme, $nrcardiapme, $nrseqcnfcpm, $cdfilauxprat, $cdprat, $cdtipoprat, $vrpeconscpm, $nrordcnfcpme);
	}
	
	private function updatePlate($plateData, $selectedPlate) {
		$updatePlate = "UPDATE COMPCARDPME SET CDFILAUXPRAT = :CDFILAUXPRAT, CDPRATO = :CDPRATO, CDTIPOPRAT = :CDTIPOPRAT, VRPECONSCPME = :VRPECONSCPME, NRORDPRACPME = :NRORDPRACPME, IDPRATREALOC = 'N' WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND NRCARDPME = :OLD_NRCARDPME AND NRCARDIAPME = :OLD_NRCARDIAPME AND NRSEQPRACPME = :OLD_NRSEQPRACPME";
		
		$params = array(
            "OLD_CDFILCENTPME" => $plateData['CDFILCENTPME'],
            "OLD_NRCARDPME"    => $plateData['NRCARDPME'],
            "OLD_NRCARDIAPME"  => $plateData['NRCARDIAPME'],
            "OLD_NRSEQPRACPME" => $plateData['NRSEQCNFCPME'],
            "CDFILAUXPRAT"     => $selectedPlate['CDFILAUXPRAT'],
            "CDPRATO"          => $selectedPlate['CDPRATO'],
            "CDTIPOPRAT"       => $plateData['CDTIPOPRAT'],
            "VRPECONSCPME"     => $plateData['VRPECONSCPME'],
            "NRORDPRACPME"     => $plateData['NRORDCNFCPME']
        );
        
        $stmt = $this->connection->prepare($updatePlate);
        $stmt->execute($params);
	}
	
	private function getConfig($plateData) {
		$sql = "SELECT C.NRSEQCNFCPME, C.NRORDCNFCPME FROM CONFCARDPME C
					WHERE C.CDFILCENTPME =:P_CDFILCENTPME AND C.CDSERVFILI =:P_CDSERVFILI AND C.NRCARDPME =:P_NRCARDPME
						AND C.CDTIPOPRAT = :P_CDTIPOPRAT AND C.NRSEQCNFCPME = :P_NRSEQCNFCPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $plateData['CDFILCENTPME'],
            "P_NRCARDPME"    => $plateData['NRCARDPME'],
            "P_CDSERVFILI"   => $plateData['CDSERVFILI'],
            "P_CDTIPOPRAT"   => $plateData['CDTIPOPRAT'],
            "P_NRSEQCNFCPME" => $plateData['NRSEQCNFCPME'],
        );
      return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* VERIFICA SE O MESMO POSSUI PRATO NESTE TIPO DE PRATO E DATA. */
	private function hasPlate($plateData) {
		$sql = "SELECT CDFILCENTPME FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND NRCARDIAPME = :P_NRCARDIAPME AND NRSEQPRACPME = :P_NRSEQPRACPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $plateData['CDFILCENTPME'],
            "P_NRCARDPME"    => $plateData['NRCARDPME'],
            "P_NRCARDIAPME"  => $plateData['NRCARDIAPME'],
            "P_NRSEQPRACPME" => $plateData['NRSEQCNFCPME']
		);
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function updateCardia($plateData) {
		// if ((isset($plateData['QTCOMESTCPME']) == false || $plateData['QTCOMESTCPME'] == 0 || !$plateData['QTCOMESTCPME']) && !($plateData['filterData']['aggregated']))  {
  //          $filterData['filterData'] = array(
  //              "cdfilcentpme"   => $plateData['CDFILCENTPME'],
  //              "aggregated"     => array('CDFILIAL' => $plateData['CDFILIAL']),
  //              "service"        => array('CDSERVFILI' => $plateData['CDSERVFILI']),
  //              "typeMenu"       => array('CDTIPOCARD' => $plateData['filterData']['typeMenu']['CDTIPOCARD']),
  //              "CDFILIALINSERT" => $plateData['filterData']['aggregated'] != null ? $plateData['filterData']['aggregated']['CDFILIAL'] : NULL
  //          );
  //          $filterData['DTCARDIAPME'] = substr($plateData['DTCARDIAPME'], 0, 10);
  //          $diners = $this->elaboracaoCardapio->getDinersCard($filterData);
  //          $qtcomen = $this->elaboracaoCardapio->getComenOfDay($diners[0], $plateData['WEEKDAY']);
  //          $this->updateCardiaInFilial($plateData, $qtcomen);
  //      } else {
            $this->utilitiesCard->updateCardiaPme(
                'S', $plateData['CDFILCENTPME'], $plateData['NRCARDPME'], $plateData['NRCARDIAPME'], 
                $plateData['CDSERVFILI'], substr($plateData['DTCARDIAPME'], 0, 10), $plateData['filterData']['aggregated']['CDFILIAL']);
        // }
        $this->updateCentralDiner($plateData);
	}
	
	// ATUALIZA CARDIAPME DA UNIDADE ESPECÍFICA
	// private function updateCardiaInFilial($plateData, $qtcomen) {
	// 	$updateCardia = "UPDATE CARDIAPME SET /*CDSERVFILI = :CDSERVFILI, CDFILIAL = :CDFILIAL, DTCARDIAPME = :DTCARDIAPME*/
	// 						QTCOMESTCPME = :QTCOMESTCPME, QTCLIESTCPME = :QTCLIESTCPME, QTFUNESTCPME = :QTFUNESTCPME, 
	// 						QTDOAESTCPME = :QTDOAESTCPME, QTCOMEFECPME = :QTCOMEFECPME, QTCLIEFECPME = :QTCLIEFECPME, 
	// 						QTFUNEFECPME = :QTFUNEFECPME, QTDOAEFECPME = :QTDOAEFECPME, IDCARDESPEC = :IDCARDESPEC
	// 					WHERE CDFILCENTPME = :OLD_CDFILCENTPME AND NRCARDPME = :OLD_NRCARDPME AND NRCARDIAPME = :OLD_NRCARDIAPME
	// 						";
	// 	$params = array(
 //           // "CDSERVFILI"       => $plateData['CDSERVFILI'],
 //           // "CDFILIAL"         => $plateData['filterData']['aggregated']['CDFILIAL'],
 //           //"DTCARDIAPME"      => $plateData['DTCARDIAPME'],
 //           "QTCOMESTCPME"	   => $qtcomen['QTCOMESTCPME'],
 //           "QTCLIESTCPME"     => $qtcomen['QTCLIESTCPME'],
 //           "QTFUNESTCPME"     => $qtcomen['QTFUNESTCPME'],
 //           "QTDOAESTCPME"     => $qtcomen['QTDOAESTCPME'],
 //           "QTCOMEFECPME"     => 0,
 //           "QTCLIEFECPME"     => 0,
 //           "QTFUNEFECPME"     => 0,
 //           "QTDOAEFECPME"     => 0,
 //           "IDCARDESPEC"      => 'S',
 //           "OLD_CDFILCENTPME" => $plateData['CDFILCENTPME'],
 //           "OLD_NRCARDPME"    => $plateData['NRCARDPME'],
 //           "OLD_NRCARDIAPME"  => $plateData['NRCARDIAPME']
 //       );
        
 //       $stmt = $this->connection->prepare($updateCardia);
 //       $stmt->execute($params);
	// }
	
	// ATUALIZA OS COMENSAIS DA CENTRAL RETIRANDO A QUANTIDADE DA AGREGADA ESPECÍFICA
	private function updateCentralDiner($plateData) {
		$row = array(
			'CDFILCENTPME' => $plateData['CDFILCENTPME'],
			'NRCARDPME'    => $plateData['NRCARDPME'],
			'CDSERVFILI'   => $plateData['CDSERVFILI'],
			'DTCARDIAPME'  => $plateData['DTCARDIAPME']
		);
		$this->comensalEstimadoCardapio->updateComenCentral($row);
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	public function receitaUtilizada(DTO\Request\Row $request, DTO\Response $response){
	    $row = $request->getRow();
	    $sql = "SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, 0.0 AS CUSTOUNIT, 0.0 AS CUSTOTOT FROM PRODUTO PD, ( SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, SGUNIDADE AS SGUNIDRECE, QTBRUTAPR AS QTDERECE, 'P' AS TIPO FROM RECEPADR P WHERE ( CDPRATO = :P_CDPRATO ) AND ( CDFILAUXPRAT = :P_CDFILAUX ) ) RC WHERE ( RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY RC.CDPRATO, RC.CDFILAUXPRAT, PD.CDARVPROD";
	    $params = array (
	        "P_CDPRATO"      => $row['CDPRATO'],
	        "P_CDFILAUX"     => $row['CDFILAUXPRAT'],
	    );
	    
	    if ($row['IDRECEITA'] == 'T') {
	    	$sql = "SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, 0.0  AS CUSTOUNIT, 0.0 AS CUSTOTOT FROM PRODUTO PD, ( SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, SGUNIDADE AS SGUNIDRECE, QTBRUTAPRTP AS QTDERECE, 'T' AS TIPO FROM RECETPCARD T WHERE ( CDPRATO = :P_CDPRATO ) AND ( CDFILAUXPRAT = :P_CDFILAUX ) AND ( CDFILCENTPME = :P_CDFILCENTPME ) AND ( CDTIPOCARD = :P_CDTIPOCARD ) ) RC WHERE ( RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY RC.CDPRATO, RC.CDFILAUXPRAT, PD.CDARVPROD";
		    $params["P_CDFILCENTPME"] = $row['CDFILCENTPME'];
		    $params["P_CDTIPOCARD"] = $row['CDTIPOCARD'];
	    }
	    
	    if ($row['IDRECEITA'] == 'F') {
	    	$sql = "SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, 0.0 AS CUSTOUNIT, 0.0  AS CUSTOTOT FROM PRODUTO PD, ( SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, SGUNIDADE AS SGUNIDRECE, QTBRUTALO AS QTDERECE, 'F' AS TIPO FROM RECEFILI F WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDPRATO = :P_CDPRATO ) AND ( CDFILAUXPRAT = :P_CDFILAUX ) ) RC WHERE ( RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY RC.CDPRATO, RC.CDFILAUXPRAT, PD.CDARVPROD";
		    $params["P_CDFILIAL"] = $row['CDFILIAL'];
	    }
	    
	    if ($row['IDRECEITA'] == 'S') {
	    	$sql = "SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, 0.0  AS CUSTOUNIT, 0.0 AS CUSTOTOT FROM PRODUTO PD, ( SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, SGUNIDADE AS SGUNIDRECE, QTBRUTASE AS QTDERECE, 'S' AS TIPO FROM RECESERV WHERE ( CDFILIAL = :P_CDFILIAL ) AND ( CDSERVFILI = :P_CDSERVFILI ) AND ( CDPRATO = :P_CDPRATO ) AND ( CDFILAUXPRAT = :P_CDFILAUX ) ) RC WHERE ( RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY RC.CDPRATO, RC.CDFILAUXPRAT, PD.CDARVPROD";
		    $params["P_CDFILIAL"] = $row['CDFILIAL'];
		    $params["P_CDSERVFILI"] = $row['CDSERVFILI'];
	    }
	    
	    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
        $result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
	
	public function receitaUtilizadaCusto(DTO\Request\Row $request, DTO\Response $response){
	    $row = $request->getRow();
	    $sql = "SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, CDARVPROD, NMPRODUTO, SGUNIDPROD, SGUNIDRECE, QTDERECE, TIPO, CUSTOUNIT, CUSTOUNIT * QTDERECE AS CUSTOTOT FROM ( SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, CUSTO_PRODUTO(:P_CDFILICUSTO, RC.CDPRODUTO, :P_NRTIPOCUSTO, :P_DTCUSTO, SYSDATE) AS CUSTOUNIT FROM PRODUTO PD, ( SELECT RP.CDPRATO, RP.CDFILAUXPRAT, RP.CDPRODUTO, RP.SGUNIDADE AS SGUNIDRECE, RP.QTBRUTAPR AS QTDERECE, 'P' AS TIPO FROM RECEPADR RP WHERE RP.CDPRATO = :P_CDPRATO AND RP.CDFILAUXPRAT = :P_CDFILAUX ) RC WHERE RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY CDPRATO, CDFILAUXPRAT, CDARVPROD";
	    $params = array (
	        "P_CDPRATO"      => $row['CDPRATO'],
	        "P_CDFILAUX"     => $row['CDFILAUXPRAT'],
	        "P_NRTIPOCUSTO"  => $row['NRTIPOCUSTO'],
	        "P_DTCUSTO"      => $row['DTCUSTO'],
	        "P_CDFILICUSTO"  => $row['CDFILICUSTO']
	    );
	    
	    if ($row['IDRECEITA'] == 'T') {
	    	$sql = "SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, CDARVPROD, NMPRODUTO, SGUNIDPROD, SGUNIDRECE, QTDERECE, TIPO, CUSTOUNIT, CUSTOUNIT * QTDERECE AS CUSTOTOT FROM ( SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, CUSTO_PRODUTO(:P_CDFILICUSTO, RC.CDPRODUTO, :P_NRTIPOCUSTO, :P_DTCUSTO, SYSDATE) AS CUSTOUNIT FROM PRODUTO PD, ( SELECT RT.CDPRATO, RT.CDFILAUXPRAT, RT.CDPRODUTO, RT.SGUNIDADE   AS SGUNIDRECE, RT.QTBRUTAPRTP AS QTDERECE, 'T' AS TIPO FROM RECETPCARD RT WHERE RT.CDPRATO = :P_CDPRATO AND RT.CDFILAUXPRAT = :P_CDFILAUX AND RT.CDFILCENTPME = :P_CDFILCENTPME AND RT.CDTIPOCARD = :P_CDTIPOCARD ) RC WHERE RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY CDPRATO, CDFILAUXPRAT, CDARVPROD";
		    $params["P_CDFILCENTPME"] = $row['CDFILCENTPME'];
		    $params["P_CDTIPOCARD"] = $row['CDTIPOCARD'];
	    }
	    
	    if ($row['IDRECEITA'] == 'F') {
	    	$sql = "SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, CDARVPROD, NMPRODUTO, SGUNIDPROD, SGUNIDRECE, QTDERECE, TIPO, CUSTOUNIT, CUSTOUNIT * QTDERECE AS CUSTOTOT FROM ( SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, CUSTO_PRODUTO(:P_CDFILICUSTO, RC.CDPRODUTO, :P_NRTIPOCUSTO, :P_DTCUSTO, SYSDATE) AS CUSTOUNIT FROM PRODUTO PD, ( SELECT RF.CDPRATO, RF.CDFILAUXPRAT, RF.CDPRODUTO, RF.SGUNIDADE AS SGUNIDRECE, RF.QTBRUTALO AS QTDERECE, 'F' AS TIPO FROM RECEFILI RF WHERE RF.CDFILIAL = :P_CDFILIAL AND RF.CDPRATO = :P_CDPRATO AND RF.CDFILAUXPRAT = :P_CDFILAUX ) RC WHERE RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY CDPRATO, CDFILAUXPRAT, CDARVPROD";
		    $params["P_CDFILIAL"] = $row['CDFILIAL'];
	    }
	    
	    if ($row['IDRECEITA'] == 'S') {
	    	$sql = "SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, CDARVPROD, NMPRODUTO, SGUNIDPROD, SGUNIDRECE, QTDERECE, TIPO, CUSTOUNIT, CUSTOUNIT * QTDERECE AS CUSTOTOT FROM ( SELECT RC.CDPRATO, RC.CDFILAUXPRAT, RC.CDPRODUTO, PD.CDARVPROD, PD.NMPRODUTO, PD.SGUNIDADE AS SGUNIDPROD, RC.SGUNIDRECE, RC.QTDERECE, RC.TIPO, CUSTO_PRODUTO(:P_CDFILICUSTO, RC.CDPRODUTO, :P_NRTIPOCUSTO, :P_DTCUSTO, SYSDATE) AS CUSTOUNIT FROM PRODUTO PD, ( SELECT CDPRATO, CDFILAUXPRAT, CDPRODUTO, SGUNIDADE AS SGUNIDRECE, QTBRUTASE AS QTDERECE, 'S' AS TIPO FROM RECESERV WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND CDPRATO = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUX )  RC WHERE RC.CDPRODUTO = PD.CDPRODUTO ) ORDER BY CDPRATO, CDFILAUXPRAT, CDARVPROD";
		    $params["P_CDFILIAL"] = $row['CDFILIAL'];
		    $params["P_CDSERVFILI"] = $row['CDSERVFILI'];
	    }
	    
	    $stmt = $this->connection->prepare($sql);
		$stmt->execute($params);
        $result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("result", $result));
	}
}