<?php
namespace PME\Controllers\Cardapio;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ElaboracaoCardapio extends \PME\Services\TeknisaCrud {
	protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $connection;
	protected $newCode;
	protected $environment;
	protected $entityManager;
	protected $lista;
	protected $utilitiesCardapio; 

    public function __construct($crudService, $parameterBag, $connection, $newCode, $environment, $entityManager, $lista, $utilitiesCardapio) {
        parent::__construct($crudService, $parameterBag);
        $this->connection = $connection;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->lista = $lista;
        $this->utilitiesCardapio = $utilitiesCardapio;
	}
	
	/* ------------------------ VERIFICA AS REGRAS QUE UM CARDAPIO PRECISAR TER PARA SER INSERIDO ------------------------*/ 
	public function menuRules(DTO\Request\Row $request, DTO\Response $response) {
		$dataCard = $request->getRow();
		$message = array(
				'isCorrect' => false,
				'message' => ''
			);
		
		try {
			$menuExtent = $this->menuExtent($dataCard);
			if (count($menuExtent) > 0) {
				$message['message'] = 'O período informado está compreendido em um período já cadastrado';
				return $response->addDataSet(new DataSet("message", $message));
			}
			$config = $this->configService($dataCard);
			if (count($config) > 0) {
				$paramPlaneja = $this->getParamPlaneja($dataCard['cdfilcentpme']);
				if ($paramPlaneja[0]['IDCUSPADCAR'] === 'S') {
					$costDefault = $this->costDefault($dataCard);
					if (count($costDefault) == 0 || $costDefault[0]['VRCUSPADTCAR'] === 0) {
						$message['message'] = 'Serviço não possui custo padrão cadastrado';
						return $response->addDataSet(new DataSet("message", $message));
					}
				} 
				if (!$paramPlaneja) {
					$message['message'] = 'Operação BLoqueada! Central sem parâmetros do planejamento cadastrado.';
					return $response->addDataSet(new DataSet("message", $message));
				}
			} else {
				$message['message'] = 'Não existe configuração de cardápio cadastrada para o serviço informado';
				return $response->addDataSet(new DataSet("message", $message));
			}
			// }
			$message['isCorrect'] = true;
			return $response->addDataSet(new DataSet("message", $message));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	// Verifica se ja possui dados no banco com a data selecionada.
	private function menuExtent($dataCard) {
		$sql = "SELECT CDFILCENTPME, DTINICARDPME, DTFINCARDPME, CDTIPOCARD FROM CARDAPIOPME, ( SELECT MIN(DTINICARDPME) DTINI, MAX(DTFINCARDPME) DTFIN FROM CARDAPIOPME WHERE CDFILCENTPME =:P_CDFILCENTPME AND CDTIPOCARD =:P_CDTIPOCARD AND DTINICARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD AND DTFINCARDPME BETWEEN :P_DTINICARD AND :P_DTFINCARD ) INTERVALO WHERE CDFILCENTPME =:P_CDFILCENTPME AND CDTIPOCARD =:P_CDTIPOCARD AND (NRCARDPME <> :P_NRCARDPME OR :P_NRCARDPME IS NULL) AND ( (:P_DTINICARD BETWEEN DTINICARDPME AND DTFINCARDPME OR :P_DTFINCARD BETWEEN DTINICARDPME AND DTFINCARDPME ) OR ( DTINICARDPME BETWEEN DTINI AND DTFIN OR DTFINCARDPME BETWEEN DTINI AND DTFIN ) )";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_DTINICARD"    => $dataCard['initialDate'],
            "P_DTFINCARD"    => $dataCard['finalDate'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME']
        );
      
      return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// Valida se o serviço tem configuração realizada.
	private function configService($dataCard) {
		$sql = "SELECT CDFILCENTPME, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR FROM CONFPADTCARD WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND CDSERVFILI = :P_CDSERVFILI";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// Valida se custo parametrizado.
	private function costDefault($dataCard) {
		$sql = "SELECT A.DTREFCUSTCAR, A.VRCUSPADTCAR, A.NRORG FROM (SELECT MAX(DTREFCUSTCAR) AS DTREFCUSTCAR, VRCUSPADTCAR, NRORG FROM CUSTPADTCARD WHERE (CDFILCENTPME =  :P_CDFILCENTPME) AND (CDTIPOCARD   =  :P_CDTIPOCARD)   AND (CDSERVFILI   =  :P_CDSERVFILI)   AND (DTREFCUSTCAR <= :P_DTINIPERIODO) GROUP BY VRCUSPADTCAR, NRORG) A";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_DTINIPERIODO" => $dataCard['initialDate']
        );
       
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* ----------------------------------------------------------------------------*/
	
	/* ------------------------ MONTA OS DADOS DO CARDAPIO ------------------------*/ 
	public function getDataCard(DTO\Request\Row $request, DTO\Response $response) {
		$dataCard = $request->getRow();

		try {
			$comens = $this->getComens($dataCard);
			$dishes = $this->getDishes($dataCard);
			$contentPlate = $this->getContentPlate($dataCard);
			$dates = $this->getMenuDate($dataCard);
			$preview = $this->getPreview($dataCard);
			$colors = ($dataCard['aggregated'] != null) ? $this->getColors($dataCard) : [];
			
			$allData = array(
				'comens'  => $comens,
				'dishes'  => $dishes,
				'content' => $contentPlate,
				'dates'   => $dates,
				'colors'  => $colors,
				'preview' => $preview,
			);
			
			$response->addDataSet(new DataSet("allData", $allData));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function getMenuDate($dataCard) {
		$sql = "SELECT DT.DTDATA   AS DTCARDIAPME, CUSTPADTCARD.VRCUSPADTCAR, DECODE(PARAMPLANEJA.IDCARDIAUTIL, 'S', FERIFILI.DTFERIFILI) AS DTFERIFILI, FERIFILI.DSFERIFILI, PARAMPLANEJA.NRCORNUTIL FROM DATA DT LEFT JOIN FERIFILI ON ( FERIFILI.CDFILIAL =:P_CDFILIAL ) AND ( FERIFILI.DTFERIFILI = DT.DTDATA ) INNER JOIN PARAMPLANEJA ON PARAMPLANEJA.CDFILIAL =:P_CDFILIAL AND PARAMPLANEJA.CDPARPLANEJA = '1' LEFT JOIN CUSTPADTCARD ON ( CDFILCENTPME =:P_CDFILCENTPME ) AND ( CDTIPOCARD =:P_CDTIPOCARD ) AND ( CDSERVFILI =:P_CDSERVFILI ) AND ( DTREFCUSTCAR = ( SELECT MAX(DTREFCUSTCAR) FROM CUSTPADTCARD WHERE CDFILCENTPME =:P_CDFILCENTPME AND CDTIPOCARD =:P_CDTIPOCARD AND CDSERVFILI =:P_CDSERVFILI AND DTREFCUSTCAR <= DTDATA ) ) WHERE ( DT.DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL ) AND ((CUSTPADTCARD.VRCUSPADTCAR IS NOT NULL AND PARAMPLANEJA.IDCUSPADCAR = 'S') OR PARAMPLANEJA.IDCUSPADCAR = 'N') ORDER BY DT.DTDATA";
		$paramSelect = array(
			"P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
			"P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
			"P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
			"P_CDTIPOCARD"   => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_DTINICIAL"    => $dataCard['initialDate'],
            "P_DTFINAL"      => $dataCard['finalDate']
        );
		// return ''; 
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getColors($dataCard) {
		$sql = "SELECT PARAMPLANEJA.NRCORNALTE, PARAMPLANEJA.NRCORNUTIL, PARAMPLANEJA.IDCARDIAUTIL, PARAMPLANEJA.IDSOMACOMEST, PARAMGERAL.QTDIABLOQCAR, NVL(PARAMPLANEJA.NRCORCARDESP, 16711680) AS NRCORCARDESP, PARAMGERAL.IDPESQPRATNOME FROM PARAMPLANEJA, PARAMGERAL WHERE PARAMPLANEJA.CDFILIAL = :P_CDFILIAL AND PARAMPLANEJA.CDPARPLANEJA = '1'";
		$paramSelect = array(
            "P_CDFILIAL" => $dataCard['aggregated']['CDFILIAL']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getDishes($dataCard) {
		$sql = "SELECT C.CDFILCENTPME, C.CDSERVFILI, C.NRCARDPME, C.NRSEQCNFCPME, C.CDTIPOPRAT, T.NMTIPOPRAT, C.IDOPCCNFCPME, C.NRORDCNFCPME FROM CONFCARDPME C, TIPOPRAT T WHERE C.CDFILCENTPME =:P_CDFILCENTPME AND C.CDSERVFILI =:P_CDSERVFILI AND C.NRCARDPME =:P_NRCARDPME AND C.CDTIPOPRAT = T.CDTIPOPRAT ORDER BY NRORDCNFCPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getComens($dataCard) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, IDCARDESPEC, NRORG FROM CARDIAPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI AND CDFILIAL = :P_CDFILIAL AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL ORDER BY DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
            "P_DTINICIAL"    => $dataCard['initialDate'],
            "P_DTFINAL"      => $dataCard['finalDate']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	private function getContentPlate($dataCard) {
		$sql = "SELECT CARD.CDFILCENTPME, CARD.NRCARDPME, CARD.NRCARDIAPME, :P_CDFILIAL AS CDFILIAL, CARD.CDSERVFILI, 
				  DT.DTDATA AS DTCARDIAPME, C.NRORDCNFCPME, C.NRSEQCNFCPME, C.CDTIPOPRAT, T.NMTIPOPRAT, C.IDOPCCNFCPME, 
				  C.IDIMPCNFCPME, CARD.NRSEQPRACPME, CARD.NRORDPRACPME, CARD.CDFILAUXPRAT, CARD.CDPRATO, 
				  CARD.NMPRATO, DECODE(CARD.IDTIPO, 'P',  CARD.VRPECONSCPME + (NVL(PP.VRPERACEIT, 0)), CARD.VRPECONSCPME) VRPECONSCPME, 
				  RECEITA_UTILIZADA(CARD.CDPRATO, CARD.CDFILAUXPRAT, :P_CDFILIAL, :P_CDSERVFILI, CARD.CDFILCENTPME, :P_CDTIPOCARD) REC_UTIL,
				  IDPRATREALOC
				 FROM DATA DT 
				 LEFT JOIN CONFCARDPME C ON C.CDFILCENTPME = :P_CDFILCENTPME AND C.NRCARDPME = :P_NRCARDPME AND C.CDSERVFILI = :P_CDSERVFILI 
				INNER JOIN TIPOPRAT T ON C.CDTIPOPRAT = T.CDTIPOPRAT 
				 LEFT JOIN (SELECT CC.CDFILCENTPME, CC.NRCARDPME, CC.NRCARDIAPME, :P_CDFILIAL AS CDFILIAL, CD.CDSERVFILI,
				                   CD.DTCARDIAPME, CC.NRSEQPRACPME, CC.NRORDPRACPME, CC.CDFILAUXPRAT, CC.CDPRATO, PT.NMPRATO,
				                   CC.CDTIPOPRAT, CC.VRPECONSCPME, CD.IDTIPO, CC.IDPRATREALOC
				              FROM PRATO PT, COMPCARDPME CC, 
				                   (SELECT 'P' IDTIPO, CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI 
				                      FROM CARDIAPME P 
				                     WHERE (P.CDFILCENTPME = :P_CDFILCENTPME) AND (P.NRCARDPME = :P_NRCARDPME) 
				                       AND (P.CDSERVFILI = :P_CDSERVFILI) AND (P.CDFILIAL = P.CDFILCENTPME) 
				                       AND NOT EXISTS(SELECT CDFILIAL FROM CARDIAPME C 
				                                       WHERE (C.CDFILCENTPME = P.CDFILCENTPME) AND (C.NRCARDPME = P.NRCARDPME) 
				                                         AND (C.CDSERVFILI = P.CDSERVFILI) AND (C.CDFILIAL = :P_CDFILIAL) 
				                                         AND (C.DTCARDIAPME = P.DTCARDIAPME) AND (C.IDCARDESPEC = 'S') )
				                     UNION ALL 
				                    SELECT 'E' IDTIPO, CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDFILIAL, DTCARDIAPME, CDSERVFILI 
				                      FROM CARDIAPME 
				                     WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND (NRCARDPME = :P_NRCARDPME) 
				                       AND (CDSERVFILI = :P_CDSERVFILI) AND (CDFILIAL = :P_CDFILIAL) AND (IDCARDESPEC = 'S')) CD 
				               WHERE (CC.CDFILCENTPME = CD.CDFILCENTPME) AND (CC.NRCARDPME = CD.NRCARDPME) 
				                 AND (CC.NRCARDIAPME = CD.NRCARDIAPME) AND (PT.CDFILAUXPRAT = CC.CDFILAUXPRAT) 
				                 AND (PT.CDPRATO = CC.CDPRATO) ORDER BY 5, 7) CARD ON DT.DTDATA = CARD.DTCARDIAPME 
				                                                                  AND CARD.CDTIPOPRAT = C.CDTIPOPRAT 
				                                                                  AND CARD.NRSEQPRACPME = C.NRSEQCNFCPME AND CARD.NRORDPRACPME = C.NRORDCNFCPME
				LEFT JOIN PERACEITPRATO PP ON PP.CDFILCENTPME = :P_CDFILCENTPME AND
				                                    PP.CDFILIAL = :P_CDFILIAL_PERACEIT AND
				                                    PP.CDPRATO = CARD.CDPRATO AND
				                                    PP.CDFILAUXPRAT = CARD.CDFILAUXPRAT AND
				                                    PP.CDTIPOCARD = :P_CDTIPOCARD
				WHERE DT.DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL";
		$paramSelect = array(
            "P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
            "P_CDTIPOCARD"	 => $dataCard['typeMenu']['CDTIPOCARD'],
            "P_NRCARDPME"    => $dataCard['NRCARDPME'],
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
            "P_CDFILIAL_PERACEIT" => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : '',
            "P_DTINICIAL"         => $dataCard['initialDate'],
            "P_DTFINAL"           => $dataCard['finalDate']
        );
        
        $plates = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        
        $colorsPlates = array();
        
        // itera cada prato para buscar a cor
		foreach($plates as &$plate) {
			$key = $plate['CDFILAUXPRAT']."_".$plate['CDPRATO']."_".$plate['REC_UTIL'];
			// checa se a cor do prato já foi buscada
			if (!isset($colorsPlates[$key])) {
				$jumpPlate = false;
				// monta a consulta da cor com base na receita utilizada do prato
				switch($plate['REC_UTIL']) {
					case 'S':
						$tableName = 'PRATSERV';
						$whereClause = 'AND (CDFILIAL = :P_CDFILIAL) AND (CDSERVFILI = :P_CDSERVFILI)';
						$columnColor = 'NRCORPRISERV';
						$paramsColor = array(
							"P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
							"P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
						);
						break;
					case 'F':
						$tableName = 'PRATFILI';
						$whereClause = 'AND (CDFILIAL = :P_CDFILIAL)';
						$columnColor = 'NRCORPRIFILI';
						$paramsColor = array(
							"P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
						);
						break;
					case 'T':
						$tableName = 'PRATTPCARD';
						$whereClause = 'AND (CDFILCENTPME = :P_CDFILCENTPME) AND (CDTIPOCARD   = :P_CDTIPOCARD)';
						$columnColor = 'NRCORPRIPRATTP';
						$paramsColor = array(
							"P_CDFILCENTPME" => $dataCard['cdfilcentpme'],
							"P_CDTIPOCARD"	 => $dataCard['typeMenu']['CDTIPOCARD'],
						);
						break;
					case 'P':
						$tableName = 'PRATPADR';
						$whereClause = '';
						$columnColor = 'NRCORPRIPRAT';
						$paramsColor = array();
						break;
					default:
						$plate['NRCOR'] = null;
						// continue;
						$jumpPlate = true;
						break;
				}
				if (!$jumpPlate) {
					$paramsColor['P_CDPRATO'] = $plate['CDPRATO'];
					$paramsColor['P_CDFILAUXPRAT'] = $plate['CDFILAUXPRAT'];
					$sqlColor = "SELECT $columnColor FROM $tableName WHERE CDPRATO = :P_CDPRATO AND CDFILAUXPRAT = :P_CDFILAUXPRAT $whereClause";
					$color = $this->entityManager->getConnection()->fetchAssoc($sqlColor, $paramsColor);
					if ($color !== false) {
						$plate['NRCOR'] = $color[$columnColor];
						$colorsPlates[$key] = $color[$columnColor];
					} else {
						$plate['NRCOR'] = null;
						$colorsPlates[$key] = null;
					}
				}
			} else {
				// se a cor do prato já estiver no array repete para o mesmo prato
				$plate['NRCOR'] = $colorsPlates[$key];
			}
        }
        
        return $plates;
	}
	
	private function getPreview($dataCard) {
		$sql = "SELECT CDFILIAL, CDSERVFILI, CDOPERADOR, NRVISUALCARD FROM PAROPERSERV WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND CDOPERADOR = :P_CDOPERADOR";
		$paramSelect = array(
            "P_CDSERVFILI"   => $dataCard['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
            "P_CDOPERADOR"   => $this->environment->getCdOperador()
        );
        
        $pv = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        if (count($pv) == 0) $this->insertStandardPreview($dataCard);
        return $pv;
	}
	
	private function insertStandardPreview($dataCard) {
		$inserCard = "INSERT INTO PAROPERSERV (CDFILIAL, CDSERVFILI, CDOPERADOR, NRVISUALCARD, NRORG, IDATIVO) VALUES (:CDFILIAL, :CDSERVFILI, :CDOPERADOR, :NRVISUALCARD, :NRORG, :IDATIVO)";
		$paramsTitleMenu = array(
	        "CDFILIAL" => $dataCard['aggregated'] != null ? $dataCard['aggregated']['CDFILIAL'] : $dataCard['cdfilcentpme'],
	        "CDSERVFILI"    => $dataCard['service']['CDSERVFILI'],
	        "CDOPERADOR"    => $this->environment->getCdOperador(),
	        "NRVISUALCARD" => 3,
	        "NRORG" => $this->environment->getNrOrg(),
	        "IDATIVO"   => 'S'
	    );
	
	    $stmt = $this->connection->prepare($inserCard);
	    $stmt->execute($paramsTitleMenu);
	}
	
	public function updatePreview(DTO\Request\Row $request, DTO\Response $response) {
		$dataCard = $request->getRow();
		$updatePlates = "UPDATE PAROPERSERV SET NRVISUALCARD = :NRVISUALCARD WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND CDOPERADOR = :P_CDOPERADOR";
		$params = array(
			"NRVISUALCARD"	=> $dataCard['preview'],
	        "P_CDFILIAL"    => $dataCard['filterData']['aggregated'] != null ? $dataCard['filterData']['aggregated']['CDFILIAL'] : $dataCard['filterData']['cdfilcentpme'],
            "P_CDSERVFILI"  => $dataCard['filterData']['service']['CDSERVFILI'],
            "P_CDOPERADOR"  => $this->environment->getCdOperador()
	    );
	    
	    $stmt = $this->connection->prepare($updatePlates);
		$stmt->execute($params);
	}
	
	/* ----------------------------------------------------------------------------*/
	
	/* ------------------------ INSERE UM NOVO CARDAPIO E UMA NOVA CONFIGURACAO  ------------------------*/
	public function insertNewCard(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	    	$this->connection->beginTransaction();
	    	$params = $request->getRow();
	    	$cdfilcentpme = $params['cdfilcentpme'];
	      
	    	$inserCard = "INSERT INTO CARDAPIOPME (CDFILCENTPME, NRCARDPME, DSCARDPME, DTINICARDPME, DTFINCARDPME, CDTIPOCARD) VALUES (:CDFILCENTPME, :NRCARDPME, :DSCARDPME, :DTINICARDPME, :DTFINCARDPME, :CDTIPOCARD)";
	    	$nrorg = $this->environment->getNrOrg();
	      
	    	$NRCARDPME = ($params['NRCARDPME'] == null) ? $this->newCode->getCodeByTableName('CARDAPIOPME'.$cdfilcentpme, $nrorg) : $params['NRCARDPME'];
	    	if ($params['NRCARDPME'] == null) {
	    		$paramsTitleMenu = array(
	                "CDFILCENTPME" => $cdfilcentpme,
	                "NRCARDPME"    => $NRCARDPME,
	                "DSCARDPME"    => $params['describeCard'],
	                "DTINICARDPME" => $params['initialDate'],
	                "DTFINCARDPME" => $params['finalDate'],
	                "CDTIPOCARD"   => $params['typeMenu']['CDTIPOCARD']
	            );
            
	            $stmt = $this->connection->prepare($inserCard);
	            $stmt->execute($paramsTitleMenu);
	    	}
	    	
         $this->insertConfigCard($params, $cdfilcentpme, $NRCARDPME); 
         $this->connection->commit();
         $response->addDataSet(new DataSet("NRCARDPME", array($NRCARDPME)));
	    } catch (\Exception $e) {
	    	$this->connection->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function insertConfigCard($request, $cdfilcentpme, $NRCARDPME) {
		$configCard = "INSERT INTO CONFCARDPME (CDFILCENTPME, NRCARDPME, CDSERVFILI, NRSEQCNFCPME, CDTIPOPRAT, IDOPCCNFCPME, IDIMPCNFCPME, NRORDCNFCPME) SELECT CDFILCENTPME, :P_NRCARDPME, CDSERVFILI, NRSEQCNFTCAR, CDTIPOPRAT, IDOPCCNFTCAR, IDIMPCNFTCAR, NRORDCNFTCAR FROM CONFPADTCARD WHERE CDFILCENTPME = :P_CDFILCENTPME AND CDTIPOCARD = :P_CDTIPOCARD AND CDSERVFILI = :P_CDSERVFILI";
		
		$paramsConfigCard = array(
            "P_CDFILCENTPME" => $cdfilcentpme,
            "P_NRCARDPME"    => $NRCARDPME,
            "P_CDSERVFILI"   => $request['service']['CDSERVFILI'],
            "P_CDTIPOCARD"   => $request['typeMenu']['CDTIPOCARD']
        );
        $stmt = $this->connection->prepare($configCard);
        $stmt->execute($paramsConfigCard);
	}
	/*------------------------------------------------------------------------*/
	
	/* ------------------------ INSERE DADOS DO PRATO SEGUINDO AS REGRAS ------------------------*/
	public function insertInCardia($row) {
		$row['filterData']['CDFILIALINSERT'] = $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : NULL;
		$cardInDay = $this->checkCardInDay($row);
		if (count($cardInDay) == 0) {
			$this->newCardia($row, 'N');
			$cardInDay = $this->checkCardInDay($row);
			
			if($row['filterData']['aggregated'] == null && $row['IDCARDESPEC'] == 'N') $this->insertAggregated($row);
		}
		
		return $cardInDay;
	}
	
	// VERIFICA SE POSSUI CARDAPIO NO DIA SELECIONADO NA CELULA.
	public function checkCardInDay($row, $central = null) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME FROM CARDIAPME
					WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI
						AND CDFILIAL = :P_CDFILIAL AND TRUNC(DTCARDIAPME) = TRUNC(TO_DATE(:P_DTCARDIAPME, 'DD/MM/YYYY'))";
		if ($central) $filial = $central;
		else $filial = $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'];
		
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_NRCARDPME"    => $row['filterData']['NRCARDPME'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_CDFILIAL"     => $filial,
            "P_DTCARDIAPME"  => substr($row['DTCARDIAPME'], 0, 10)
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	public function newCardia($row, $dinerScreen) {
		$newCardia = "INSERT INTO CARDIAPME (CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME, QTCOMESTCPME, QTCLIESTCPME, QTFUNESTCPME, QTDOAESTCPME, QTCOMEFECPME, QTCLIEFECPME, QTFUNEFECPME, QTDOAEFECPME, IDCARDESPEC) VALUES (:CDFILCENTPME, :NRCARDPME, :NRCARDIAPME, :CDSERVFILI, :CDFILIAL, :DTCARDIAPME, :QTCOMESTCPME, :QTCLIESTCPME, :QTFUNESTCPME, :QTDOAESTCPME, :QTCOMEFECPME, :QTCLIEFECPME, :QTFUNEFECPME, :QTDOAEFECPME, :IDCARDESPEC)";
		// $this->connection->beginTransaction();
		$holiday = $this->isHoliday($row);
		$holidayInCard = $this->isHolidayInCard($row);
		$paramPlaneja = $this->getParamPlaneja($row['filterData']['cdfilcentpme']);
		
		$diners = $this->getDinersCard($row);
		
		// VERIFICA SE O CARDAPIO POSSUI FERIADO
		if(count($holiday) != 0 && count($holidayInCard) != 0 && $paramPlaneja[0]['IDCARDIAUTIL'] != 'N') {
			$row['WEEKDAY'] = 7;
		}
		
		$qtcomen = $this->getComenOfDay($diners[0], $row['WEEKDAY']);
		if ($qtcomen['QTCOMESTCPME'] > 0 || $dinerScreen !== 'N') {
			$nrorg = $this->environment->getNrOrg();
			$newNrcardiapme  = $this->newCode->getCodeByTableName('CARDIAPME'.$row['filterData']['cdfilcentpme'].$row['filterData']['NRCARDPME'], $nrorg);
			
			$insertNewCardia = array(
					"CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
					"NRCARDPME"    => $row['filterData']['NRCARDPME'],
					"NRCARDIAPME"  => $newNrcardiapme,
					"CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
					"CDFILIAL"     => $row['filterData']['CDFILIALINSERT'] != NULL ? $row['filterData']['CDFILIALINSERT'] : $row['filterData']['cdfilcentpme'],
					"DTCARDIAPME"  => $row['DTCARDIAPME'],
					"QTCOMESTCPME" => $qtcomen['QTCOMESTCPME'], 
					"QTCLIESTCPME" => $qtcomen['QTCLIESTCPME'],
					"QTFUNESTCPME" => $qtcomen['QTFUNESTCPME'], 
					"QTDOAESTCPME" => $qtcomen['QTDOAESTCPME'],
					"QTCOMEFECPME" => 0, 
					"QTCLIEFECPME" => 0, 
					"QTFUNEFECPME" => 0, 
					"QTDOAEFECPME" => 0,
					"IDCARDESPEC"  => $row['filterData']['aggregated'] == null ? 'N' : 'S'
			);
			
			$stmt = $this->connection->prepare($newCardia);
			$stmt->execute($insertNewCardia);
		}
	}
	
	// PEGA AS COMENSAIS DA CENTRAL (AGREGADA OU CARDAPIO ESPECIFICO)
	public function getDinersCard($row) {
		$sql = "SELECT A.QTCOMPMESEG, A.QTCOMPMETER, A.QTCOMPMEQUA, A.QTCOMPMEQUI, A.QTCOMPMESEX, A.QTCOMPMESAB, A.QTCOMPMEDOM, A.QTCOMPMEFER, A.QTCLIPMESEG, A.QTCLIPMETER, A.QTCLIPMEQUA, A.QTCLIPMEQUI, A.QTCLIPMESEX, A.QTCLIPMESAB, A.QTCLIPMEDOM, A.QTCLIPMEFER, A.QTFUNPMESEG, A.QTFUNPMETER, A.QTFUNPMEQUA, A.QTFUNPMEQUI, A.QTFUNPMESEX, A.QTFUNPMESAB, A.QTFUNPMEDOM, A.QTFUNPMEFER, A.QTDOAPMESEG, A.QTDOAPMETER, A.QTDOAPMEQUA, A.QTDOAPMEQUI, A.QTDOAPMESEX, A.QTDOAPMESAB, A.QTDOAPMEDOM, A.QTDOAPMEFER FROM ( SELECT SUM(QTCOMPMESEG) AS QTCOMPMESEG, SUM(QTCOMPMETER) AS QTCOMPMETER, SUM(QTCOMPMEQUA) AS QTCOMPMEQUA, SUM(QTCOMPMEQUI) AS QTCOMPMEQUI, SUM(QTCOMPMESEX) AS QTCOMPMESEX, SUM(QTCOMPMESAB) AS QTCOMPMESAB, SUM(QTCOMPMEDOM) AS QTCOMPMEDOM, SUM(QTCOMPMEFER) AS QTCOMPMEFER, SUM(QTCLIPMESEG) AS QTCLIPMESEG, SUM(QTCLIPMETER) AS QTCLIPMETER, SUM(QTCLIPMEQUA) AS QTCLIPMEQUA, SUM(QTCLIPMEQUI) AS QTCLIPMEQUI, SUM(QTCLIPMESEX) AS QTCLIPMESEX, SUM(QTCLIPMESAB) AS QTCLIPMESAB, SUM(QTCLIPMEDOM) AS QTCLIPMEDOM, SUM(QTCLIPMEFER) AS QTCLIPMEFER, SUM(QTFUNPMESEG) AS QTFUNPMESEG, SUM(QTFUNPMETER) AS QTFUNPMETER, SUM(QTFUNPMEQUA) AS QTFUNPMEQUA, SUM(QTFUNPMEQUI) AS QTFUNPMEQUI, SUM(QTFUNPMESEX) AS QTFUNPMESEX, SUM(QTFUNPMESAB) AS QTFUNPMESAB, SUM(QTFUNPMEDOM) AS QTFUNPMEDOM, SUM(QTFUNPMEFER) AS QTFUNPMEFER, SUM(QTDOAPMESEG) AS QTDOAPMESEG, SUM(QTDOAPMETER) AS QTDOAPMETER, SUM(QTDOAPMEQUA) AS QTDOAPMEQUA, SUM(QTDOAPMEQUI) AS QTDOAPMEQUI, SUM(QTDOAPMESEX) AS QTDOAPMESEX, SUM(QTDOAPMESAB) AS QTDOAPMESAB, SUM(QTDOAPMEDOM) AS QTDOAPMEDOM, SUM(QTDOAPMEFER) AS QTDOAPMEFER FROM COMESERVTCAR C INNER JOIN FILITIPOCARD F ON ( F.CDFILCENTPME = C.CDFILCENTPME ) AND ( F.CDTIPOCARD = C.CDTIPOCARD ) AND ( F.CDFILIAL = C.CDFILIAL ) AND ( F.IDTCARATFILI = 'S' ) INNER JOIN FILIAL FL ON C.CDFILIAL = FL.CDFILIAL AND FL.CDFILCENTPME = C.CDFILCENTPME INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE ( C.CDFILCENTPME = :P_CDFILCENTPME ) AND ( C.CDTIPOCARD = :P_CDTIPOCARD ) AND ( C.CDSERVFILI = :P_CDSERVFILI ) AND ( C.CDFILIAL = :P_CDFILIAL OR (:P_CDFILIAL IS NULL AND NOT EXISTS( SELECT CARD.NRCARDPME FROM CARDIAPME CARD INNER JOIN CARDAPIOPME CP ON CP.CDFILCENTPME = CARD.CDFILCENTPME AND CP.NRCARDPME = CARD.NRCARDPME WHERE CARD.CDFILCENTPME = C.CDFILCENTPME AND CARD.CDFILIAL = C.CDFILIAL AND CARD.CDSERVFILI = C.CDSERVFILI AND CP.CDTIPOCARD = C.CDTIPOCARD AND CARD.IDCARDESPEC = 'S' AND CARD.DTCARDIAPME = :P_DTCARDIAPME)) ) ) A";
		$paramSelect = array(
			"P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
			"P_CDFILIAL"     => $row['filterData']['CDFILIALINSERT'] != null ? $row['filterData']['CDFILIALINSERT'] : null,
			"P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
			"P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD'],
			"P_DTCARDIAPME"  => substr($row['DTCARDIAPME'], 0, 10)
		);
		/* $sql = "SELECT A.QTCOMPMESEG, A.QTCOMPMETER, A.QTCOMPMEQUA, A.QTCOMPMEQUI, A.QTCOMPMESEX, A.QTCOMPMESAB, A.QTCOMPMEDOM, A.QTCOMPMEFER, A.QTCLIPMESEG, A.QTCLIPMETER, A.QTCLIPMEQUA, A.QTCLIPMEQUI, A.QTCLIPMESEX, A.QTCLIPMESAB, A.QTCLIPMEDOM, A.QTCLIPMEFER, A.QTFUNPMESEG, A.QTFUNPMETER, A.QTFUNPMEQUA, A.QTFUNPMEQUI, A.QTFUNPMESEX, A.QTFUNPMESAB, A.QTFUNPMEDOM, A.QTFUNPMEFER, A.QTDOAPMESEG, A.QTDOAPMETER, A.QTDOAPMEQUA, A.QTDOAPMEQUI, A.QTDOAPMESEX, A.QTDOAPMESAB, A.QTDOAPMEDOM, A.QTDOAPMEFER FROM ( SELECT SUM(QTCOMPMESEG) AS QTCOMPMESEG, SUM(QTCOMPMETER) AS QTCOMPMETER, SUM(QTCOMPMEQUA) AS QTCOMPMEQUA, SUM(QTCOMPMEQUI) AS QTCOMPMEQUI, SUM(QTCOMPMESEX) AS QTCOMPMESEX, SUM(QTCOMPMESAB) AS QTCOMPMESAB, SUM(QTCOMPMEDOM) AS QTCOMPMEDOM, SUM(QTCOMPMEFER) AS QTCOMPMEFER, SUM(QTCLIPMESEG) AS QTCLIPMESEG, SUM(QTCLIPMETER) AS QTCLIPMETER, SUM(QTCLIPMEQUA) AS QTCLIPMEQUA, SUM(QTCLIPMEQUI) AS QTCLIPMEQUI, SUM(QTCLIPMESEX) AS QTCLIPMESEX, SUM(QTCLIPMESAB) AS QTCLIPMESAB, SUM(QTCLIPMEDOM) AS QTCLIPMEDOM, SUM(QTCLIPMEFER) AS QTCLIPMEFER, SUM(QTFUNPMESEG) AS QTFUNPMESEG, SUM(QTFUNPMETER) AS QTFUNPMETER, SUM(QTFUNPMEQUA) AS QTFUNPMEQUA, SUM(QTFUNPMEQUI) AS QTFUNPMEQUI, SUM(QTFUNPMESEX) AS QTFUNPMESEX, SUM(QTFUNPMESAB) AS QTFUNPMESAB, SUM(QTFUNPMEDOM) AS QTFUNPMEDOM, SUM(QTFUNPMEFER) AS QTFUNPMEFER, SUM(QTDOAPMESEG) AS QTDOAPMESEG, SUM(QTDOAPMETER) AS QTDOAPMETER, SUM(QTDOAPMEQUA) AS QTDOAPMEQUA, SUM(QTDOAPMEQUI) AS QTDOAPMEQUI, SUM(QTDOAPMESEX) AS QTDOAPMESEX, SUM(QTDOAPMESAB) AS QTDOAPMESAB, SUM(QTDOAPMEDOM) AS QTDOAPMEDOM, SUM(QTDOAPMEFER) AS QTDOAPMEFER FROM COMESERVTCAR C INNER JOIN FILITIPOCARD F ON ( F.CDFILCENTPME = C.CDFILCENTPME ) AND ( F.CDTIPOCARD = C.CDTIPOCARD ) AND ( F.CDFILIAL = C.CDFILIAL ) AND ( F.IDTCARATFILI = 'S' ) INNER JOIN FILIAL FL ON C.CDFILIAL = FL.CDFILIAL AND FL.CDFILCENTPME = C.CDFILCENTPME INNER JOIN SERVICO S ON S.CDFILIAL = C.CDFILIAL AND S.CDSERVFILI = C.CDSERVFILI WHERE ( C.CDFILCENTPME = :P_CDFILCENTPME ) AND ( C.CDTIPOCARD = :P_CDTIPOCARD ) AND ( C.CDSERVFILI = :P_CDSERVFILI ) AND ( ( C.CDFILIAL = :P_CDFILIAL OR :P_CDFILIAL IS NULL ) ) ) A";
		$paramSelect = array(
				"P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
				"P_CDFILIAL"     => $row['filterData']['CDFILIALINSERT'] != null ? $row['filterData']['CDFILIALINSERT'] : null,
				"P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
				"P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD']
		);
		*/
    return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// VERIFICA SE NO DIA QUE ESTÁ SENDO INSERINDO O PRATO É UM FERIADO (PRA CENTRAL OU AGREGADA ESPECÍFICA)
	private function isHoliday($row) {
		$sql = "SELECT DTFERIFILI, DSFERIFILI FROM FERIFILI WHERE CDFILIAL = :P_CDFILIAL AND DTFERIFILI = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILIAL" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// VERIFICA SE NO DIA QUE ESTÁ SENDO INSERINDO O PRATO É UM FERIADO NO CARDÁPIO (PRA CENTRAL OU AGREGADA ESPECÍFICA)
	private function isHolidayInCard($row) {
		$sql = "SELECT DTFERICARD, DSFERICARD FROM FERICARD WHERE CDFILIAL = :P_CDFILIAL AND DTFERICARD = :P_DTCARDIAPME";
		$paramSelect = array(
            "P_CDFILIAL" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_DTCARDIAPME"  => $row['DTCARDIAPME']
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	// USE DAY OF THE WEEK, WITH SUNDAY AS 0 AND SATURDAY AS 6.
	public function getComenOfDay($diners, $day) {
		switch($day) {
			case 0: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEDOM'], "QTCLIESTCPME" => $diners['QTCLIPMEDOM'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEDOM'], "QTDOAESTCPME" => $diners['QTDOAPMEDOM']		
	        	);
	        	break;
        	case 1: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESEG'], "QTCLIESTCPME" => $diners['QTCLIPMESEG'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESEG'], "QTDOAESTCPME" => $diners['QTDOAPMESEG']		
	        	);
	        	break;
        	case 2: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMETER'], "QTCLIESTCPME" => $diners['QTCLIPMETER'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMETER'], "QTDOAESTCPME" => $diners['QTDOAPMETER']		
	        	);
	        	break;
        	case 3: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEQUA'], "QTCLIESTCPME" => $diners['QTCLIPMEQUA'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEQUA'], "QTDOAESTCPME" => $diners['QTDOAPMEQUA']		
	        	);
	        	break;
        	case 4: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEQUI'], "QTCLIESTCPME" => $diners['QTCLIPMEQUI'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEQUI'], "QTDOAESTCPME" => $diners['QTDOAPMEQUI']		
	        	);
	        	break;
        	case 5: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESEX'], "QTCLIESTCPME" => $diners['QTCLIPMESEX'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESEX'], "QTDOAESTCPME" => $diners['QTDOAPMESEX']		
	        	);
	        	break;
        	case 6: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMESAB'], "QTCLIESTCPME" => $diners['QTCLIPMESAB'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMESAB'], "QTDOAESTCPME" => $diners['QTDOAPMESAB']		
	        	);
	        	break;
        	case 7: 
				return array(
		        	"QTCOMESTCPME" => $diners['QTCOMPMEFER'], "QTCLIESTCPME" => $diners['QTCLIPMEFER'], 
		            "QTFUNESTCPME" => $diners['QTFUNPMEFER'], "QTDOAESTCPME" => $diners['QTDOAPMEFER']		
	        	);
	        	break;
		}
	}
	
	private function insertAggregated($row) {
		$aggregates = $this->getAggregated($row);
		foreach($aggregates as $aggregated) {
			$hasUnity = $this->UnityHaveCard($aggregated['CDFILIAL'], $row);
			if(count($hasUnity) == 0) {
				$row['filterData']['CDFILIALINSERT'] = $aggregated['CDFILIAL'];
				$this->newCardia($row, 'S'); // parâmetro 'dinerScreen' alterado 'N' -> 'S': adicionar os comensais ao 'getCardiaOnClickPlate' (DadosDoPrato).
			}
		}
	}
	
	/* BUSCA AS AGREGADAS QUE POSSUEM O TIPO DE CARDÁPIO ATIVO */
	private function getAggregated($row) {
		// $sql = "SELECT DISTINCT F.CDFILIAL FROM FILIAL F INNER JOIN FILITIPOCARD T ON ( T.CDFILCENTPME = F.CDFILCENTPME ) AND ( T.CDFILIAL = F.CDFILIAL ) AND ( INSTR(:CDTIPOCARD, T.CDTIPOCARD) > 0 OR :CDTIPOCARD = 'T' ) AND ( T.IDTCARATFILI = 'S' ) LEFT JOIN FILGRUPFILI    FL ON ( FL.CDGRUPFILI = :CDGRUPFILI OR :CDGRUPFILI = 'T' ) AND FL.CDFILIAL = F.CDFILIAL INNER JOIN SERVICO        S ON S.CDFILIAL = F.CDFILIAL AND S.CDSERVFILI = :CDSERVFILI WHERE ( FL.CDGRUPFILI IS NOT NULL OR :CDGRUPFILI = 'T' ) AND F.CDFILCENTPME = :CDFILCENTPME ORDER BY CDFILIAL";
		$sql = "SELECT FILITIPOCARD.CDFILIAL, FILITIPOCARD.NRORG FROM FILITIPOCARD INNER JOIN FILIAL ON FILIAL.CDFILCENTPME = FILITIPOCARD.CDFILCENTPME AND FILIAL.CDFILIAL = FILITIPOCARD.CDFILIAL INNER JOIN SERVTIPOCARD ON ( SERVTIPOCARD.CDFILCENTPME = FILITIPOCARD.CDFILCENTPME ) AND ( SERVTIPOCARD.CDTIPOCARD = FILITIPOCARD.CDTIPOCARD ) AND ( SERVTIPOCARD.CDSERVFILI =:P_CDSERVFILI ) INNER JOIN SERVICO ON ( SERVICO.CDFILIAL = FILITIPOCARD.CDFILIAL ) AND ( SERVICO.CDSERVFILI = SERVTIPOCARD.CDSERVFILI ) INNER JOIN PARAMPLANEJA ON ( FILITIPOCARD.CDFILIAL = PARAMPLANEJA.CDFILIAL ) WHERE ( FILITIPOCARD.CDFILCENTPME =:P_CDFILCENTPME ) AND FILITIPOCARD.CDTIPOCARD = :P_CDTIPOCARD AND ( FILITIPOCARD.IDTCARATFILI = 'S' ) ORDER BY FILITIPOCARD.CDFILIAL";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['aggregated'] != null ? $row['filterData']['aggregated']['CDFILIAL'] : $row['filterData']['cdfilcentpme'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_CDTIPOCARD"   => $row['filterData']['typeMenu']['CDTIPOCARD']
            // , "CDGRUPFILI"   => 'T'
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* VERIFICA SE TEM CARDÁPIO PARA ESTA UNIDADE */
	private function UnityHaveCard($cdfilial, $row) {
		$sql = "SELECT CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME FROM CARDIAPME
					WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI 
						AND CDFILIAL = :P_CDFILIAL AND TRUNC(DTCARDIAPME) = TRUNC(TO_DATE(:P_DTCARDIAPME, 'DD/MM/YYYY'))";
		$paramSelect = array(
            "P_CDFILCENTPME" => $row['filterData']['cdfilcentpme'],
            "P_NRCARDPME"    => $row['filterData']['NRCARDPME'],
            "P_CDSERVFILI"   => $row['filterData']['service']['CDSERVFILI'],
            "P_DTCARDIAPME"  => substr($row['DTCARDIAPME'], 0, 10),
            "P_CDFILIAL"     => $cdfilial
        );
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	/* ------------------------ CHAMA A PROCEDURE ATUALIZA_COMENSAL ------------------------*/
	
	public function callUpdateComen(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		try {
			$nrorg = $this->environment->getNrOrg();
			if (\Zeedhi\Framework\Util\Functions::arrayKeyExists('aggregatedObjects', $row)) {
				$allAggregateds = $row['aggregatedObjects'];
				$aggregatedsInactives = [];
				foreach($allAggregateds as $unity) {
					if ($unity['IDTCARATFILI'] !== 'S') {
						$aggregatedsInactives[] = $unity;
					}
				}
				$aggregatedsInactivesConcat = implode("_", array_column($aggregatedsInactives, 'CDFILIAL'));
				if (!empty($aggregatedsInactives)) {
					$this->zeraComenAggregateInactive(array(
						'P_CDFILIAL' => $aggregatedsInactivesConcat,
						'P_CDFILCENTPME' => $row['cdfilcentpme'],
						'P_DATAFIN' => $row['finalDate'],
						'P_DATAINI' => $row['initialDate'],
						'P_CDTIPOCARD' => $row['typeMenu']['CDTIPOCARD'],
						'P_CDSERVFILI' => $row['service']['CDSERVFILI'],
					));
				}
			}
			$nrlista = $this->lista->geraLista($row['aggregated']['CDFILIAL'], $nrorg);
			$this->procedureUpdateComen($row, $nrlista);
			$response->addDataSet(new DataSet("updated", array(true)));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function procedureUpdateComen($row, $lista) {
		$connection = $this->entityManager->getConnection();
    	$procedureCons = new StoredProcedure($connection, 'ATUALIZA_COMENSAL');
    	
		$procedureCons->addParam(new Param('P_CDFILCENTPME', Param::PARAM_INPUT, $row['cdfilcentpme'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_NRCARDPME',    Param::PARAM_INPUT, $row['NRCARDPME'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_NRLISTA',      Param::PARAM_INPUT, $lista,  Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDSERVFILI',   Param::PARAM_INPUT, $row['service']['CDSERVFILI'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_CDTIPOCARD',   Param::PARAM_INPUT, $row['typeMenu']['CDTIPOCARD'],   Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTINI',        Param::PARAM_INPUT, $row['DATAPROCE'], Param::PARAM_TYPE_STR));
    	$procedureCons->addParam(new Param('P_DTFIN',        Param::PARAM_INPUT, $row['finalDate'],   Param::PARAM_TYPE_STR));
    	
    	$procedureCons->execute();
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	/* FUNÇÃO PARA PEGAR OS DADOS DA PARAMPLANEJA (PODE ADICIONAR MAIS COLUNAS FUTURAMENTE) */
	private function getParamPlaneja($cdfilial) {
		$sql = "SELECT IDCARDIAUTIL, IDCUSPADCAR, IDPERCCOME FROM PARAMPLANEJA WHERE CDFILIAL = :P_CDFILIAL AND CDPARPLANEJA = '1'";
		$paramSelect = array(
            "P_CDFILIAL" => $cdfilial
        );
        
        return $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
	}
	
	public function hasConfigCard(DTO\Request\Row $request, DTO\Response $response) {
		$sql = "SELECT * FROM CONFCARDPME WHERE NRCARDPME = :NRCARDPME AND CDSERVFILI = :CDSERVFILI AND CDFILCENTPME = :CDFILCENTPME";
		$hasData = false;
		$row = $request->getRow();
		
		$paramSelect = array(
            "NRCARDPME"    => $row['NRCARDPME'],
            "CDSERVFILI"   => $row['service']['CDSERVFILI'],
            "CDFILCENTPME" => $row['cdfilcentpme']
        );
        
        $paramPlaneja = $this->getParamPlaneja($row['cdfilcentpme']);
        $data = $this->entityManager->getConnection()->fetchAll($sql, $paramSelect);
        
        $params = array(
        	'hasData' => (count($data) > 0),
        	'IDPERCCOME' => $paramPlaneja[0]['IDPERCCOME']
        );
        
        $response->addDataSet(new DataSet("hasConfigCard", $params));
	}
	
	/* ------------------------------------------------------------------------------------------------ */
	
	public function saveCardapioDescription(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$updateCard = "UPDATE CARDAPIOPME SET DSCARDPME = :DSCARDPME WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND NRCARDPME = :NRCARDPME ";
		$params = array(
			"DSCARDPME"	=> $row['DSCARDPME'],
	        "CDFILCENTPME"    => $row['CDFILCENTPME'],
            "CDTIPOCARD"  => $row['CDTIPOCARD'],
            'NRCARDPME' => $row['NRCARDPME']
	    );
	    
	    $stmt = $this->connection->prepare($updateCard);
		$stmt->execute($params);
	}
	
	private function zeraComenAggregateInactive($data) {
		$query = "UPDATE CARDIAPME SET QTFUNESTCPME = 0, QTDOAESTCPME = 0, QTCLIESTCPME = 0, QTCOMESTCPME = 0 WHERE (CDFILCENTPME = :P_CDFILCENTPME) AND ( INSTR(:P_CDFILIAL, CDFILIAL) > 0 ) AND (CDSERVFILI = :P_CDSERVFILI) AND ( DTCARDIAPME BETWEEN :P_DATAINI AND :P_DATAFIN ) AND ( EXISTS ( SELECT CARDAPIOPME.NRCARDPME FROM CARDAPIOPME WHERE ( CARDAPIOPME.CDFILCENTPME = CARDIAPME.CDFILCENTPME ) AND ( CARDAPIOPME.NRCARDPME = CARDIAPME.NRCARDPME ) AND ( DTCARDIAPME BETWEEN CARDAPIOPME.DTINICARDPME AND CARDAPIOPME.DTFINCARDPME ) AND ( CARDAPIOPME.CDTIPOCARD = :P_CDTIPOCARD ) ) )";
	    $stmt = $this->connection->prepare($query);
		$stmt->execute($data);
	}
	
	public function deleteComensInDaysWithoutPlate(DTO\Request\Row $request, DTO\Response $response) {
		try {
			$row = $request->getRow();
			if ($row['aggregated'] === null) {
				$query = "DELETE FROM CARDIAPME 
					WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI 
					AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL  /*AND QTCOMESTCPME > 0*/ AND IDCARDESPEC = 'N'
					AND DTCARDIAPME NOT IN (
                    SELECT DTCARDIAPME FROM CARDIAPME CD JOIN COMPCARDPME CC 
                        ON CD.CDFILCENTPME = CC.CDFILCENTPME AND CC.NRCARDPME = CD.NRCARDPME AND CD.NRCARDIAPME = CC.NRCARDIAPME
                        WHERE CC.CDFILCENTPME = :P_CDFILCENTPME AND CC.NRCARDPME = :P_NRCARDPME
                    ) ";
                    
				$params = array(
		            "P_CDFILCENTPME" => $row['cdfilcentpme'],
		            "P_NRCARDPME"    => $row['NRCARDPME'],
		            "P_CDSERVFILI"   => $row['service']['CDSERVFILI'],
		            "P_DTINICIAL"    => $row['initialDate'],
		            "P_DTFINAL"      => $row['finalDate'],
		        );
			} else {
				$query = "DELETE FROM CARDIAPME 
					WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME AND CDSERVFILI = :P_CDSERVFILI 
					AND DTCARDIAPME BETWEEN :P_DTINICIAL AND :P_DTFINAL /* AND QTCOMESTCPME > 0*/ AND CDFILIAL = :P_CDFILIAL AND IDCARDESPEC = 'S'
					AND NRCARDIAPME NOT IN (SELECT NRCARDIAPME FROM COMPCARDPME WHERE CDFILCENTPME = :P_CDFILCENTPME AND NRCARDPME = :P_NRCARDPME) ";
					//AND QTCOMESTCPME > 0
				$params = array(
		            "P_CDFILCENTPME" => $row['cdfilcentpme'],
		            "P_NRCARDPME"    => $row['NRCARDPME'],
		            "P_CDSERVFILI"   => $row['service']['CDSERVFILI'],
		            "P_CDFILIAL"     => $row['aggregated'] != null ? $row['aggregated']['CDFILIAL'] : $row['cdfilcentpme'],
		            "P_DTINICIAL"    => $row['initialDate'],
		            "P_DTFINAL"      => $row['finalDate'],
		        );	
			}
		    $stmt = $this->connection->prepare($query);
			$stmt->execute($params);
			$response->addDataSet(new DataSet("updated", array(true)));
		} catch (\Exception $e) {
	    	$response->setError(new Error($e->getMessage(), 500));
		}
	}
}