import { Loader, Metadata, IDictionary } from '@zeedhi/core';
  
import { GridEditable, LoadingService } from '@zeedhi/common';
  
export class AlunoController {
  public gridEditController = Loader.getInstance('GridEditableController');
    
  public crudController = Loader.getInstance('CrudController');
    
  public utilities = Loader.getInstance('Utilities');
    
  public async saveAluno() {
    const grid: GridEditable = Metadata.getInstance('gridAluno');
    const editedRows = grid.getEditedRows(false, true);

    if (grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        // Http.post('/retornaCodigoCategoria', {}).then((response: IDictionary) => {
        // let codigo = response.data.dataset.codigo[0].CODIGO;
        LoadingService.show(' ');
        // await grid.datasource.get();
        let total = grid.datasource.data[grid.datasource.data.length - 1].MAX;

        editedRows.forEach((row: IDictionary) => {
          if (row.__is_new) {
            row.CDALUNO = total + 1;
            total += 1;
            row.NRMATRICULA = row.NRMATRICULA ? row.NRMATRICULA : null;
            row.DTNASCI = row.DTNASCI ? row.DTNASCI : null;
            row.DTFINATEND = row.DTFINATEND ? row.DTFINATEND : null;
          }
        });
        
        this.crudController.saveRows(editedRows, 'aluno', grid).then(() => {
          this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
        }).catch((error: string) => {
          if (error.includes('ORA-00001')) {
            this.utilities.showAlert('alert', 'Impossível adicionar. Registro já existe.', 'error');
          } else this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          this.gridEditController.defaultState(grid);
          LoadingService.hide();
        });
        // }).catch((error) => {
        //   this.utilities.showAlert('alertError', error, 'error');
        // });
      } else {
        this.gridEditController.defaultState(grid);
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'yellow');
      }
    }
  }
  
  public get filterGrid() {
    return [
      {
        name: 'DATEFORMAT',
        operator: '=',
        value: this.utilities.getDateFormat,
      },
    ];
  }
}
