import Vue from 'vue';
import Zeedhi from '@zeedhi/vue';
import ZeedhiComponents from '@zeedhi/vuetify';
import TeknisaComponents from '@zeedhi/teknisa-components-vuetify';
import TekLibComponents from '@zeedhi/tek-lib';
import { ZdAvatar } from '@zeedhi/zd-avatar';
import { ZdFavorites } from '@zeedhi/zd-favorites';
import { ZdCrosstab, ZdCrosstabEditable } from '@zeedhi/zd-crosstab';

import { Icons } from '@zeedhi/common';
import {
  Messages, KeyMap, Loader,
} from '@zeedhi/core';

import { ZdRichText } from '@zeedhi/zd-richtext';

import '@fortawesome/fontawesome-free/css/all.css';
import { mdiApplicationCog, mdiCurrencyUsdCircleOutline } from '@mdi/js';
import router from '../router';
import controllers from '../controllers';
import config from '../config/config';
import '@zeedhi/common/dist/style.css';
import '@zeedhi/vuetify/dist/zd-style.css';
import vuetify from './vuetify';
import '../styles/style.scss';
import '../controllers/utilities/MaskController';

Icons.addIcons({
  dragHandle: 'mdi-drag-vertical',
  applicationCog: mdiApplicationCog,
  usdCircleOutline: mdiCurrencyUsdCircleOutline,
  iconName: 'mdi-icon-id',
});

Vue.use(Zeedhi, {
  controllers,
  router,
  config,
});
Vue.use(ZeedhiComponents);
Vue.use(TeknisaComponents);
Vue.use(TekLibComponents);

Vue.component('ZdAvatar', ZdAvatar);
Vue.component('ZdRichText', ZdRichText);
Vue.component('ZdFavorites', ZdFavorites);
Vue.component('ZdCrosstab', ZdCrosstab);
Vue.component('ZdCrosstabEditable', ZdCrosstabEditable);

vuetify.framework.icons.values = Icons.getIcons() as any;

Messages.add({
  'pt-BR': {
    translation: {
      KEY: 'valor',
    },
  },
  'es-CL': {
    translation: {
      KEY: 'valor',
    },
  },
  'es-CO': {
    translation: {
      KEY: 'valor',
    },
  },
  'es-ES': {
    translation: {
      KEY: 'valor',
    },
  },
  'en-US': {
    translation: {
      KEY: 'value',
    },
  },
});

Messages.add({
  'pt-BR': {
    translation: {
      NO_DATA: 'Não há dados',
    },
  },
  'es-CL': {
    translation: {
      NO_DATA: 'No hay datos',
    },
  },
  'es-CO': {
    translation: {
      NO_DATA: 'Sin datos',
    },
  },
  'es-ES': {
    translation: {
      NO_DATA: 'Sin datos',
    },
  },
  'en-US': {
    translation: {
      NO_DATA: 'No data',
    },
  },
});

// moment.locale(I18n.instance.language);
// moment.extend(localeData);
// moment.extend(localizedFormat);

const appController = Loader.getInstance('AppController');

KeyMap.bind({
  esc: {
    event: () => { appController.escAction(); },
    stop: true,
  },
});
